/* $Log$
 * Revision 1.1.1.1  1999/12/30 23:46:39  stuart
 * BMS library
 *
 */

#include <time.h>
#include <date.h>

#define DAYSUPTO1900	2415385L

/* convert BMS (DATE) to BMS (JULDAY). */
JULDAY DtoJ(DATE doc) {
  if (doc == zeroD) return zeroJ;
  return doc + DAYSUPTO1900;
}

/* convert BMS (JULDAY) to BMS (DATE). */
DATE JtoD(JULDAY jday) {
  if (jday == zeroJ) return zeroD;
  return jday - DAYSUPTO1900;
}

/* convert UNIX time_t to JULDAY */
JULDAY timeJ(time_t ut) {
  struct mmddyy mdy;
  struct tm *t = localtime(&ut); /* need this to account for timezones */
  mdy.mm = t->tm_mon + 1;
  mdy.dd = t->tm_mday;
  mdy.yy = t->tm_year + 1900;
  return julian(&mdy);
}

/* FIXME: this makes 4000 a leap year (it shouldn't be).  
   Be sure to fix julmdy() as well.  Don't put it off - you have only 2000
   years to take care of this.
   NOTE: this follows the Gregorian (Catholic) calendar.  Protestants adopted
   the leap year adjustments much later, setting Sep 3 = Sep 14, 1752.  The
   unix 'cal' program follows the protestant calendar.
   FIXME: Historically, the year did not always start on Jan 01. The Jan 01
   convention was adopted at different times in different places.
   NOTE: The Julian calendar didn't settle down until the reign of 
   Ceasar Augustus, so BC calendar dates are ficticious.
 */

JULDAY julian(const struct mmddyy *mdy) {
  int dyyr,lpyrs,cyrs,clyrs;
  JULDAY dayno;

  if (!mdy) return timeJ(time(0));	/* return system date */
  if (mdy->mm == 99 && mdy->dd == 99)
    return 0x7FFFFFFF;
  dyyr=(mdy->mm*275)/9-32+mdy->dd;
  lpyrs=mdy->yy+4712;
  dayno=lpyrs*365L;
  cyrs=clyrs=0;
  if (mdy->yy>1582) { cyrs=mdy->yy-1500; clyrs=mdy->yy-1200; }
  if (mdy->mm<3)    { cyrs--; clyrs--; lpyrs--; dyyr+=2; }
  dayno += (dyyr+clyrs/400+lpyrs/4-cyrs/100);
  if (dayno>2299170L) dayno-=10;	/* Oct 5 = Oct 15, 1582 */
  return dayno;
}
