/*
   id is arbitrary.  Mutually exclusive operations must have the same id.
	Using the same id when not necessary reduces concurrency, but
	doesn't break anything.

   op is the sysV semaphore operation.  You will most likely use:

	-1	lock
	 1	unlock

   other values can be used to implement a "ball of marbles".  However,
   this should be done with a seperate routine so as not to confuse
   the locking use.
*/
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <errenv.h>

#define BMSKEY	911L
#define NSEMS	5		/* number of BMS locks */

void semlock(int id,int op) {
  static int sid = -1;	/* semaphore identifier */
  struct sembuf s;
  if (sid<0) {
    for(;;) {
      sid = semget(BMSKEY,NSEMS,0);
      if (sid>=0) break;
      sid = semget(BMSKEY,NSEMS,IPC_CREAT+IPC_EXCL+0666);
      if (sid>=0) {
	for (s.sem_num=0; s.sem_num<NSEMS; s.sem_num++) {
	  s.sem_op = 1;
	  s.sem_flg = 0;
	  semop(sid,&s,1);	/* init one at a time */
	}
	break;
      }
    }
  }
  s.sem_num = id % NSEMS;
  s.sem_op  = op;
  s.sem_flg = SEM_UNDO;
  if (semop(sid,&s,1)<0) errdesc("semop",errno);
}
