#include <date.h>
#include <ctype.h>

static const char wtbl[7][4] = {
  "SUN", "MON", "TUE", "WED", "THU", "FRI", "SAT"
};

static char mtbl[12][4] = {
  "JAN", "FEB", "MAR", "APR", "MAY", "JUN",
  "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"
};

void tm_mask(const struct tm *t,const char *mask,char *txt) {
  /* do left justified fields */
  const char *s = mask;
  int am = 0, wday = 0, mon = 0, zone = 0;
  int day, year, hour, min, sec, jul;
  while (*s) {
    int c = *s;
    switch (c) {
    case 'W': case 'w':		/* weekday name */
      c = wtbl[t->tm_wday][wday];
      if (c) ++wday; else c = ' ';
      break;
    case 'N': case 'n':		/* month name */
      c = mtbl[t->tm_mon][mon];
      if (c) ++mon; else c = ' ';
      break;
    case 'A': case 'a':
      switch (++am) {
      case 1:
	c = (t->tm_hour < 12) ? 'A' : 'P'; break;
      case 2:
	c = 'M'; break;
      default:
	c = ' ';
      }
      break;
    case 'Z': case 'z':
      c = tzname[t->tm_isdst][zone];
      if (c) ++zone; else c = ' ';
      break;
    }
    if (islower(*s++) && isupper(c))
      c = _tolower(c);
    *txt++ = c;
  }
  /* do right justified fields */
  mon = t->tm_mon + 1;
  day = t->tm_mday;
  year = t->tm_year + 1900;
  hour = t->tm_hour;
  if (am) {
    if (hour > 12)
      hour -= 12;
    else if (!hour)
      hour = 12;
  }
  min = t->tm_min;
  sec = t->tm_sec;
  jul = t->tm_yday;
  while (s > mask) {
    --txt;
    switch (*--s) {
    case 's':
      *txt = sec % 10 + '0'; sec /= 10; break;
    casem:
    case 'm':
      *txt = min % 10 + '0'; min /= 10; break;
    caseh:
    case 'h':
      *txt = hour % 10 + '0'; hour /= 10; break;
    case 'Y':
      if (year)
	*txt = year % 10 + '0', year /= 10;
      else
	*txt = ' ';
      break;
    case 'M':
      *txt = mon % 10 + '0'; mon /= 10; break;
    case 'D':
      *txt = day % 10 + '0'; day /= 10; break;
    case 'J':
      *txt = jul % 10 + '0'; jul /= 10; break;
    }
  }
}

void timemask(time_t t,const char *mask,char *txt) {
  time_t now = time(0);
  int old = 0;
  int i;
  char buf[64];
  if (t < now - 180 * 86400L || t > now + 180 * 86400L)
    old = 5;
  for (i = 0; i < sizeof buf - 1 && mask[i]; ++i) {
    int c = mask[i];
    if (c == 'C') switch (++old) {
    case 1: case 2:
      c = 'h'; break;
    case 3:
      c = ':'; break;
    case 4: case 5:
      c = 'm'; break;
    default:
      c = 'Y';
    }
    buf[i] = c;
  }
  buf[i] = 0;
  tm_mask(localtime(&t),buf,txt);
}
