/* binary tree algorithm - from Knuth Vol 3 */
#include <stdlib.h>
#include "tree.h"

#define link(a,p) ( ((a)<0) ? &(p)->llink : &(p)->rlink )

PTR *
tsearch(const PTR k,TREE *root,int (*c)(const PTR, const PTR)) {
  treenode head;
  int a;			/* current balance factor */
  tree *T = *root;
  struct treenode *p, *q;	/* primary node pointers */
  treenode *s, *t, *r;	/* subtree pointers for balancing */

  if (!T) {		/* if tree empty */
    T = malloc(sizeof(tree));
    if (T == 0) return 0;
    T->prev = 0;
    p = T->root = T->a;
    T->cnt = 1;
    *root = T;
    p->data = (PTR)k;
    p->llink = p->rlink = 0;		/* init first node */
    p->bal = 0;
    return &p->data;
  }
  else
    p = T->root;

  s = head.rlink = p;			/* A1 */
  t = &head;				/* initialize pointers */

  for (;;) {
    if ((a = (*c)(k,p->data)) == 0)	/* A2 */
      return &p->data;			/* found item */
    q = *link(a,p);			/* A3,A4 */
    if (q==0) {
      if (T->cnt >= MEMCNT) {
	tree *N = malloc(sizeof(tree));
	if (N == 0) return 0;
	N->prev = T;
	N->root = T->root;
	N->cnt = 0;
	T = *root = N;
      }
      q = &T->a[T->cnt++];
      *link(a,p) = q;
      break;				/* found where to add to tree */
    }
    if (q->bal) { t=p; s=q; }
    p = q;
  }

  q->data = (PTR)k;			/* A5 */
  q->llink = q->rlink = 0;		/* initialize new node */
  q->bal = 0;

  a = ((*c)(k,s->data)<0)?-1:1;
  r = p = *link(a,s);			/* A6 */
  while (p!=q)				/* adjust balance factors */
    p = *link(p->bal=((*c)(k,p->data)<0)?-1:1,p);

  if (s->bal == 0) {			/* A7 */
    s->bal = a;
    return &q->data;
  }					/* adjust subtree balance */
  if (s->bal == -a) {
    s->bal = 0;
    return &q->data;
  }

  if (r->bal == a) {
    p = r;				/* A8 */
    *link(a,s) = *link(-a,r);
    *link(-a,r) = s;			/* simple rebalance */
    s->bal = r->bal = 0;
  }
  else {
    p = *link(-a,r);			/* A9 */
    *link(-a,r) = *link(a,p);
    *link(a,p) = r;			/* complicated rebalance */
    *link(a,s) = *link(-a,p);
    *link(-a,p) = s;
    s->bal = r->bal = 0;
    if (p->bal == a) s->bal = -a;
    if (p->bal == -a) r->bal = a;
    p->bal = 0;
  }

  if (s == t->rlink)			/* A10 */
    t->rlink = p;			/* reroot subtree */
  else
    t->llink = p;
  T->root = head.rlink;			/* update user's root pointer */
  return &q->data;
}

PTR
tfind(const PTR k,const TREE *root,int (*c)(const PTR, const PTR)) {
  tree *T = *root;
  treenode *p;
  int a;
  if (!T) return 0;
  for (p = T->root; p; p = *link(a,p)) {
    if ((a = (*c)(k,p->data)) == 0)	/* A2 */
      return p->data;			/* found item */
  }
  return 0;
}
